package main

import (
	"fmt"
	"launchpad.net/gnuflag"
	"math/rand"
)

var verbose = gnuflag.Bool("verbose", false, "report step-by-step stuff")
var seed = gnuflag.Int64("seed", 0, "the seed for the random generation")
var doSwitch = gnuflag.Bool("switch", false, "if true, always switch your guess")
var nRounds = gnuflag.Int("rounds", 100, "set the number of rounds")

type round struct {
	door   int
	opened int
	guess1 int
	guess2 int
}

type guesser func(r *round)

func guess1(r *round) {
	r.guess1 = rand.Intn(3) + 1
}

func monty(r *round) {
	for {
		open := rand.Intn(3) + 1
		if open != r.door && open != r.guess1 {
			r.opened = open
			break
		}
	}
}

func stayer(r *round) {
	r.guess2 = r.guess1
}

func switcher(r *round) {
	guess2 := r.guess1
	for i := 0; i < 3; i++ {
		guess2 = (guess2 % 3) + 1
		if guess2 != r.guess1 && guess2 != r.opened {
			r.guess2 = guess2
			break
		}
	}
}

func oneRound(g guesser) bool {
	aRound := round{door: rand.Intn(3) + 1, guess1: 0, guess2: 0}
	guess1(&aRound)
	monty(&aRound)
	g(&aRound)
	success := aRound.guess2 == aRound.door
	if *verbose {
		//door, g1, opened, g2, success
		fmt.Printf("%d %d %d %d %t\n", aRound.door, aRound.guess1,
			aRound.opened, aRound.guess2, success)
	}
	return success

}

func main() {
	gnuflag.Parse(true)
	var g guesser
	if *doSwitch {
		g = switcher
	} else {
		g = stayer
	}
	rounds := 0
	success := 0
	rand.Seed(*seed)
	for i := 0; i < *nRounds; i++ {
		if oneRound(g) {
			success++
		}
		rounds++
	}
	fmt.Printf("After %d rounds, %d success == %.3f%% success\n",
		rounds, success, float64(success)*100.0/float64(rounds))
}
